@@ -6,7 +6,11 @@ use HTML::Template;
 use Devel::StackTrace;
 use IO::File;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
+
+our $A_HEAD = '<a class="package" href="?rm=view_code&amp;module=';
+our $A_TAIL = ';view_code_no_popup=1">';
+our $A_END  = '</a>';
 
 our $TEMPLATE = qq{
 <html><!-- HTML::Template -->
@@ -39,6 +43,7 @@ our $TEMPLATE = qq{
                margin: 4px;
                margin-bottom: 10px;
                -moz-border-radius: 10px;
+               overflow: auto;
            }
            h1 {
                margin: 0;
@@ -63,18 +68,21 @@ our $TEMPLATE = qq{
                border: 1px dotted #000;
                padding: 5px;
                margin: 8px;
-               width: 90%;
+               overflow: auto;
            }
            pre b {
                font-weight: bold;
                color: #000;
                background-color: #E68200;
            }
+           h1 a.package {
+               color: #999999;
+           }
        </style>
    </head>
    <body>
        <div class="box">
-           <h1><!-- TMPL_VAR NAME="title" --></h1>
+           <h1><!-- TMPL_VAR NAME="title_a" --></h1>
 
            <div class="url"><!-- TMPL_VAR NAME="url" --></div>
            <div class="infos">
@@ -111,7 +119,6 @@ sub import {
     my $self   = shift;
     my $caller = scalar caller;
 
-    no strict 'refs';
     $caller->add_callback( 'init', sub{
         my $self = shift;
         my $de;
@@ -119,10 +126,22 @@ sub import {
             push @{$self->{__stacktrace}},[Devel::StackTrace->new(ignore_package=>[qw/CGI::Application::Plugin::DebugScreen Carp CGI::Carp/])->frames];
             die @_; # rethrow
         };
-        *{"$caller\::report"} = \&debug_report;
+        {
+            no strict 'refs';
+            *{"$caller\::report"} = \&debug_report;
+        }
     });
+
     $caller->add_callback( 'error', sub{
         my $self = shift;
+        if (
+              exists $INC{'CGI/Application/Plugin/ViewCode.pm'}
+              &&
+            ! exists $INC{'CGI/Application/Dispatch.pm'}
+           )
+        {
+            $self->{__viewcode}++;
+        }
         $self->report(@_);
     });
 }
@@ -133,15 +152,25 @@ sub debug_report{
     my $url = $self->query->url;
     my $title = ref $self || $self;
 
+    $title = html_escape($title);
+    my $title_a = $title;
+
+    if ( $self->{__viewcode} ) {
+        $title_a = $A_HEAD . $title . $A_TAIL . $title . $A_END;
+    }
+
     my $stacks = $self->{__stacktrace}[0];
 
     my @stacktraces;
     for my $stack ( @{$stacks} ) {
         my %s;
-        $s{package}  = exists $stack->{pkg} ? $stack->{pkg}  : $stack->{package};
-        $s{filename} = $stack->{file} ? $stack->{file} : $stack->{filename};
+        $s{package}  = exists $stack->{pkg}  ? $stack->{pkg}  : $stack->{package};
+        $s{filename} = exists $stack->{file} ? $stack->{file} : $stack->{filename};
 
         $s{package}  = html_escape($s{package});
+        if ( $self->{__viewcode} && $s{package} ne 'main' ) {
+            $s{package} = $A_HEAD . $s{package} . $A_TAIL . $s{package} . $A_END;
+        }
         $s{filename} = html_escape($s{filename});
         $s{line}     = html_escape($stack->{line});
         $s{code_preview} = print_context($s{filename},$s{line});
@@ -153,9 +182,10 @@ sub debug_report{
         die_on_bad_params => 0,
     );
     $t->param(
-        title  => html_escape($title),
-        url    => html_escape($url),
-        desc   => html_escape($desc),
+        title   => $title,
+        title_a => $title_a,
+        url     => html_escape($url),
+        desc    => html_escape($desc),
         stacktrace => \@stacktraces,
     );
 
@@ -205,7 +235,7 @@ CGI::Application::Plugin::DebugScreen - add Debug support to CGI::Application.
 
 =head1 VERSION
 
-This documentation refers to CGI::Application::Plugin::DebugScreen version 0.02
+This documentation refers to CGI::Application::Plugin::DebugScreen version 0.03
 
 =head1 SYNOPSIS
 
@@ -223,6 +253,11 @@ When 'die' is generated by 'runmode',
  this plug-in outputs the stack trace by error_mode().
 The error cannot be caught excluding runmode.
 
+If CGI::Application::Plugin::ViewCode is used,
+ this uses CGI::Application::Plugin::ViewCode. 
+But CGI::Application::Dispatch is used,
+ this not uses CGI::Application::Plugin::ViewCode.
+
 =head1 DEPENDENCIES
 
 L<strict>
@@ -237,6 +272,8 @@ L<Devel::StackTrace>
 
 L<IO::File>
 
+L<CGI::Application::Plugin::ViewCode>
+
 =head1 BUGS AND LIMITATIONS
 
 There are no known bugs in this module.
@@ -257,6 +294,8 @@ L<Devel::StackTrace>
 
 L<IO::File>
 
+L<CGI::Application::Plugin::ViewCode>
+
 =head1 Thanks To
 
 MATSUNO Tokuhiro (MATSUNO)
@@ -265,6 +304,8 @@ Koichi Taniguchi (TANIGUCHI)
 
 Masahiro Nagano (KAZEBURO)
 
+Tomoyuki Misonou
+
 =head1 AUTHOR
 
 Atsushi Kobayashi, E<lt>nekokak@cpan.orgE<gt>
